<?php

namespace Drupal\o365_groups;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\o365\GraphService;
use Microsoft\Graph\Model\Channel;
use Microsoft\Graph\Model\Group;

/**
 * Service used to do all kinds of stuff with groups and teams.
 */
class GroupsService {

  /**
   * The o365.graph service.
   *
   * @var \Drupal\o365\GraphService
   */
  protected $o365Graph;

  /**
   * The cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cacheBackend;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * List of all teams.
   *
   * @var array
   */
  protected $teams = [];

  /**
   * The default endpoint for getting a list of teams.
   *
   * @var string
   */
  protected $getTeamsEndpoint = "/me/joinedTeams";

  /**
   * Constructs a GroupsService object.
   *
   * @param \Drupal\o365\GraphService $o365_graph
   *   The o365.graph service.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cacheBackend
   *   The cache backend.
   * @param \Drupal\Core\Session\AccountProxyInterface $accountProxy
   *   The account proxy.
   */
  public function __construct(GraphService $o365_graph, CacheBackendInterface $cacheBackend, AccountProxyInterface $accountProxy) {
    $this->o365Graph = $o365_graph;
    $this->cacheBackend = $cacheBackend;
    $this->currentUser = $accountProxy->getAccount();
  }

  /**
   * Get all the available groups from teams.
   *
   * @return array
   *   The list of available teams.
   */
  public function getGroupsFromTeams() {
    $this->getTeamsFromGraph();

    $teams = [];
    foreach ($this->teams as $id => $name) {
      $teams[$id] = $name;
    }

    // Sort on value.
    asort($teams);
    return $teams;
  }

  /**
   * Get a list of all groups in Office 365.
   *
   * @param false|string $endpoint
   *   The endpoint string containing the skipToken or FALSE for default.
   *
   * @throws \Drupal\Core\TempStore\TempStoreException
   * @throws \GuzzleHttp\Exception\GuzzleException
   * @throws \League\OAuth2\Client\Provider\Exception\IdentityProviderException
   */
  protected function getTeamsFromGraph($endpoint = FALSE) {
    $cid = 'getTeamsFromGraph::' . $this->currentUser->id();
    $cache = $this->cacheBackend->get($cid);
    $teams = [];

    if (!$cache) {
      try {
        if (!$endpoint) {
          $endpoint = $this->getTeamsEndpoint;
        }

        $data = $this->o365Graph->getGraphData($endpoint, 'GET', FALSE, FALSE, Group::class);

        if ($data) {
          /** @var \Microsoft\Graph\Model\Group $group */
          foreach ($data as $group) {
            $channelEndpoint = '/teams/' . $group->getId() . '/channels';
            $groupName = $group->getDisplayName();

            $channels = $this->o365Graph->getGraphData($channelEndpoint, 'GET', FALSE, FALSE, Channel::class);
            if ($channels) {
              /** @var \Microsoft\Graph\Model\Channel $channel */
              foreach ($channels as $channel) {
                $key = $group->getId() . '|||' . $channel->getId();
                $teams[$key] = $groupName . ' - ' . $channel->getDisplayName();
              }
            }
          }
        }

        $expireDate = strtotime('+4 hours');
        $this->cacheBackend->set($cid, $teams, $expireDate);
      }
      finally {
        $this->teams = $teams;
      }
    }
    else {
      $this->teams = $cache->data;
    }
  }

  /**
   * Generate a link to a team channel based on the ID.
   *
   * @param string $id
   *   The channel ID.
   *
   * @return string|false
   *   The URL to the team or FALSE.
   *
   * @throws \Drupal\Core\TempStore\TempStoreException
   * @throws \GuzzleHttp\Exception\GuzzleException
   * @throws \League\OAuth2\Client\Provider\Exception\IdentityProviderException
   */
  public function generateTeamChannelLink($id) {
    if ($this->o365Graph->getCurrentUserId()) {
      // Explode the ID on the 3 pipes.
      $ids = explode('|||', $id);

      if (count($ids) > 1) {
        $endpoint = '/teams/' . $ids[0] . '/channels/' . $ids[1];
        /** @var \Microsoft\Graph\Model\Channel $channel */
        $channel = $this->o365Graph->getGraphData($endpoint, 'GET', FALSE, FALSE, Channel::class);

        if ($channel) {
          return $channel->getWebUrl();
        }
      }
    }

    return FALSE;
  }

}
